
allcause.ROCt<-function(times, failures, variable, pro.time, cut.off, estimator="naive", lambda=NULL) {

if(sum(estimator!=c("kaplan-meier", "akritas", "naive"))==3)
{ stop("The estimator have to be selected among the three following possibilities: 'kaplan-meier', 'akritas', 'naive'.") }

if(!is.null(lambda) & estimator!="akritas")
{ stop("The parameter lambda should be only defined for the Akritas estimator.") }

if(is.null(lambda) & estimator=="akritas")
{ stop("The parameter lambda have to be defined by the user.") }

if (estimator=="kaplan-meier") {

.temp.data <- data.frame(times=times, failures=failures, variable=variable)

.n.na<-sum(is.na(.temp.data$times + .temp.data$failures + .temp.data$variable))

.temp.data <- .temp.data[!is.na(.temp.data$times + .temp.data$failures + .temp.data$variable),]

.surv.total <- summary(survfit(Surv(times, failures) ~ 1, data = .temp.data))

.surv.total$indic <- (.surv.total$time<=pro.time)

.temp.se<-function(x) {

   .surv.cond<-try(summary(survfit(Surv(times, failures) ~ 1,
      data=.temp.data[.temp.data$variable>x,])), silent = TRUE)
   
  if(inherits(.surv.cond, "try-error")) {return(NA)}

  else {
  
  .surv.cond <- data.frame(time = c(1-0.99999999999, .surv.cond$time),
     surv = c(0.99999999999, .surv.cond$surv))
  .surv.cond$surv[.surv.cond$surv==0] <- 1-0.99999999999
  .surv.cond$indic <- (.surv.cond$time<=pro.time)
  
   return( (1-.surv.cond$surv[.surv.cond$indic][sum(.surv.cond$indic)]) * 
           (1-(sum(.temp.data$variable<=x)/length(times))) / 
           (1-.surv.total$surv[.surv.total$indic][sum(.surv.total$indic)]) ) } }
  
.temp.sp<-function(x) {

   .surv.cond<-try(summary(survfit(Surv(times, failures) ~ 1,
     data=.temp.data[.temp.data$variable<=x,])), silent = TRUE)
   
  if(inherits(.surv.cond, "try-error")) {return(NA)}

  else {
  
  .surv.cond <- data.frame(time = c(1-0.99999999999, .surv.cond$time),
     surv = c(0.99999999999, .surv.cond$surv))
  .surv.cond$surv[.surv.cond$surv==0] <- 1-0.99999999999
  .surv.cond$indic <- (.surv.cond$time<=pro.time)
  
    return( (.surv.cond$surv[.surv.cond$indic][sum(.surv.cond$indic)]) *
	(sum(.temp.data$variable<=x)/length(times)) /
    .surv.total$surv[.surv.total$indic][sum(.surv.total$indic)]) }  }

.tab.res<-data.frame(
   cut.off=cut.off,
   se=suppressWarnings(sapply(cut.off, FUN=".temp.se")),
   sp1=1-suppressWarnings(sapply(cut.off, FUN=".temp.sp")))

.tab.res$se[.tab.res$se>1]<-NA
.tab.res$sp1[.tab.res$sp1>1]<-NA

.tab.res<-.tab.res[!is.na(.tab.res$sp1 + .tab.res$se),]

.tab.res.temp<-.tab.res[!is.na(.tab.res$sp1 + .tab.res$se),]

.tab.res.temp<-.tab.res.temp[order(.tab.res.temp$sp1, .tab.res.temp$se),]

.tab.res.temp<-rbind(c(NA, 0, 0), .tab.res.temp, c(NA, 1, 1))

.tab.res$sp<-1-.tab.res$sp1

return(list(
table = .tab.res[,c("cut.off", "se", "sp")],
auc = sum((.tab.res.temp$sp1[2:length(.tab.res.temp$sp1)]-.tab.res.temp$sp1[1:(length(.tab.res.temp$sp1)-1)])*(.tab.res.temp$se[2:length(.tab.res.temp$se)])),
missing = .n.na  ) ) }

if (estimator=="akritas") {

.temp.data <- data.frame(times=times, failures=failures, variable=variable)

.n.na<-sum(is.na(.temp.data$times + .temp.data$failures + .temp.data$variable))
   
.temp.data <- .temp.data[!is.na(.temp.data$times + .temp.data$failures + .temp.data$variable),]

.temp.data <- .temp.data[order(.temp.data$variable),]

.pas <- round(lambda * length(times))

.n <- dim(.temp.data)[1]

.survie.x<-function(x)  { 
  .tmp<-summary(survfit(Surv(times, failures)~1 ,
     data=.temp.data[max(1, (x-.pas)):min((x+.pas), .n),]))
  min(0.99999, .tmp$surv[.tmp$time<=pro.time][sum(.tmp$time<=pro.time)])  }

.survie.temp<-sapply(1:.n, FUN = ".survie.x")

.survie.lambda<-function(x) {
 sum(.survie.temp * (.temp.data$variable>x)) / .n }

.survie.marginale<-sum(.survie.temp) / .n

.temp.se<-function(x) {
 (1-sum(.temp.data$variable<=x)/.n-.survie.lambda(x))/(1-.survie.marginale) }
 
.temp.sp<-function(x)
{ 1-(.survie.lambda(x)/.survie.marginale) }

.tab.res<-data.frame(
   cut.off=cut.off,
   se=sapply(cut.off, FUN=".temp.se"),
   sp1=1-sapply(cut.off, FUN=".temp.sp"))

.tab.res$se[.tab.res$se>1]<-NA
.tab.res$sp1[.tab.res$sp1>1]<-NA

.tab.res<-.tab.res[!is.na(.tab.res$sp1 + .tab.res$se),]

.tab.res.temp<-.tab.res[!is.na(.tab.res$sp1 + .tab.res$se),]

.tab.res.temp<-.tab.res.temp[order(.tab.res.temp$sp1, .tab.res.temp$se),]

.tab.res.temp<-rbind(c(NA, 0, 0), .tab.res.temp, c(NA, 1, 1))

.tab.res$sp<-1-.tab.res$sp1

return(list(
table = .tab.res[,c("cut.off", "se", "sp")],
auc = sum((.tab.res.temp$sp1[2:length(.tab.res.temp$sp1)] - .tab.res.temp$sp1[1:(length(.tab.res.temp$sp1)-1)]) * 0.5 * (.tab.res.temp$se[2:length(.tab.res.temp$sp1)] + .tab.res.temp$se[1:(length(.tab.res.temp$sp1)-1)])),
missing = .n.na  ) ) }


if (estimator=="naive") {

.temp.data <- data.frame(times=times, failures=failures, variable=variable)

.n.na<-sum(is.na(.temp.data$times + .temp.data$failures + .temp.data$variable))
   
.temp.data <- .temp.data[!is.na(.temp.data$times + .temp.data$failures + .temp.data$variable),]

km.c <- summary(survfit(Surv(.temp.data$times, .temp.data$failures==0)~1))
km.c <- data.frame(time=c(0, km.c$time), surv=c(1, km.c$surv))
km.c$surv[km.c$surv==0] <- 0.0001
s.c <- 1 / (sapply(.temp.data$times, FUN=function(x) {km.c$surv[km.c$time<=x][sum(km.c$time<=x)]}))

se <- function(x) {
sum(1*((.temp.data$variable > cut.off[x]) & (.temp.data$times <= pro.time))*.temp.data$failures*s.c)/sum(1*(.temp.data$times <= pro.time)*.temp.data$failures*s.c) }

sp <- function(x) {
 sum(1*((.temp.data$variable <= cut.off[x]) & (.temp.data$times > pro.time)))/sum(1*(.temp.data$times > pro.time)) }

temp.se <- sapply(1:length(cut.off), FUN = "se")
temp.sp <- sapply(1:length(cut.off), FUN = "sp")

temp.sp1 <- 1-temp.sp

.tab.res <-data.frame(
 cut.off = c(min(.temp.data$variable), cut.off, max(.temp.data$variable)),
 sp = c(0, temp.sp, 1),
 se = c(1, temp.se, 0))
 
.tab.res$sp1 <- 1-.tab.res$sp

.tab.res <- .tab.res[order(.tab.res$sp1, .tab.res$se),]

.auc <- sum((.tab.res$sp1[2:length(.tab.res$sp1)] - .tab.res$sp1[1:(length(.tab.res$sp1)-1)]) * 0.5 * (.tab.res$se[2:length(.tab.res$sp1)] + .tab.res$se[1:(length(.tab.res$sp1)-1)]))

return(list(
table = .tab.res[,c("cut.off", "se", "sp")],
auc = .auc,
missing = .n.na  ) ) }

}



net.ROCt<-function(times, failures, variable, p.age, p.sex, p.year, rate.table, pro.time, cut.off, knn=FALSE, lambda=NULL) {

.warnings <- "none"

.temp.data <- data.frame(times=times, failures=failures, variable=variable, age=p.age, sex=p.sex, year=p.year)

.n.na<-sum(is.na(.temp.data$times + .temp.data$failures + .temp.data$variable +
   .temp.data$age + 1*(.temp.data$sex=="male") + .temp.data$year))

.temp.data <- .temp.data[!is.na(.temp.data$times + .temp.data$failures + .temp.data$variable + .temp.data$age + 1*(.temp.data$sex=="male") + .temp.data$year),]
   
.rs.model <- try(summary(rs.surv(Surv(times, failures)~ 1 + ratetable(age=age, sex=sex, year=year),  ratetable=rate.table, data=.temp.data, method="pohar-perme")), silent = TRUE)

  if(inherits(.rs.model, "try-error")) {return(NA)}
  
  else {
  
  if(sum(.rs.model$surv>1)>0) { .warnings <- "Esimations of the net survival are higher than 1." }
  
  
if (knn==FALSE) {

.surv.total <- data.frame(time = c(1-0.99999999999, .rs.model$time),
     surv = c(0.99999999999, .rs.model$surv))
.surv.total$surv[.surv.total$surv==0] <- 1-0.99999999999
.surv.total$indic <- (.surv.total$time<=pro.time)
  
.temp.se<-function(x) {

  .rs.model.cond <- try(summary(rs.surv(Surv(times, failures)~ 1 + ratetable(age=age, sex=sex, year=year),  ratetable=rate.table, data=.temp.data[.temp.data$variable>x,], method="pohar-perme")), silent = TRUE)
  
  if(inherits(.rs.model.cond, "try-error")) {return(NA)}

  else {
  
  .surv.cond <- data.frame(time = c(1-0.99999999999, .rs.model.cond$time),
     surv = c(0.99999999999, .rs.model.cond$surv))
  .surv.cond$surv[.surv.cond$surv==0] <- 1-0.99999999999
  .surv.cond$indic <- (.surv.cond$time<=pro.time)

   return( (1-.surv.cond$surv[.surv.cond$indic][sum(.surv.cond$indic)]) * 
           (1-(sum(.temp.data$variable<=x)/length(times))) / 
           (1-.surv.total$surv[.surv.total$indic][sum(.surv.total$indic)]) ) } }
		   
.temp.sp<-function(x) {
  
  .rs.model.cond <- try(summary(rs.surv(Surv(times, failures)~ 1 + ratetable(age=age, sex=sex, year=year),  ratetable=rate.table, data=.temp.data[.temp.data$variable<=x,], method="pohar-perme")), silent = TRUE)

  if(inherits(.rs.model.cond, "try-error")) {return(NA)}

  else {
  
  .surv.cond <- data.frame(time = c(1-0.99999999999, .rs.model.cond$time),
     surv = c(0.99999999999, .rs.model.cond$surv))
  .surv.cond$surv[.surv.cond$surv==0] <- 1-0.99999999999
  .surv.cond$indic <- (.surv.cond$time<=pro.time)

    return( (.surv.cond$surv[.surv.cond$indic][sum(.surv.cond$indic)]) *
	        (sum(.temp.data$variable<=x)/length(times)) /
            .surv.total$surv[.surv.total$indic][sum(.surv.total$indic)] ) } }

.tab.res<-data.frame(
   cut.off=cut.off,
   se=suppressWarnings(sapply(cut.off, FUN=".temp.se")),
   sp1=1-suppressWarnings(sapply(cut.off, FUN=".temp.sp")))

.tab.res$se[.tab.res$se>1]<-NA
.tab.res$sp1[.tab.res$sp1>1]<-NA

.tab.res<-.tab.res[!is.na(.tab.res$sp1 + .tab.res$se),]

.tab.res.temp<-.tab.res[!is.na(.tab.res$sp1 + .tab.res$se),]

.tab.res.temp<-.tab.res.temp[order(.tab.res.temp$sp1, .tab.res.temp$se),]

.tab.res.temp<-rbind(c(NA, 0, 0), .tab.res.temp, c(NA, 1, 1))

.tab.res$sp<-1-.tab.res$sp1

return(list(
table = .tab.res[,c("cut.off", "se", "sp")],
auc = sum((.tab.res.temp$sp1[2:length(.tab.res.temp$sp1)]-.tab.res.temp$sp1[1:(length(.tab.res.temp$sp1)-1)])*(.tab.res.temp$se[2:length(.tab.res.temp$se)])),
missing = .n.na  ) ) }


if (knn==TRUE) {

.temp.data <- data.frame(times=times, failures=failures, variable=variable,
   age = p.age, sex = p.sex, year = p.year)

.n.na<-sum(is.na(.temp.data$times + .temp.data$failures + .temp.data$variable +
   .temp.data$age + 1*(.temp.data$sex=="male") + .temp.data$year))
   
.temp.data <- .temp.data[!is.na(.temp.data$times + .temp.data$failures + .temp.data$variable + .temp.data$age + 1*(.temp.data$sex=="male") + .temp.data$year),]
   
.temp.data <- .temp.data[order(.temp.data$variable),]

.pas <- round(lambda * dim(.temp.data)[1])

.n <- dim(.temp.data)[1]

.survie.x<-function(x)  {

   .tmp<-try(summary(rs.surv(Surv(times, failures)~ 1 + ratetable(age=age, sex=sex, year=year),  ratetable=rate.table, data=.temp.data[max(1, (x-.pas)):min((x+.pas), .n),], method="pohar-perme")), silent = TRUE)

  if(inherits(.tmp, "try-error")) {return(NA)}

  else {
  
  .tmp <- data.frame(time = c(1-0.99999999999, .tmp$time),
     surv = c(0.99999999999, .tmp$surv))
  .tmp$surv[.tmp$surv==0] <- 1-0.99999999999
  .tmp$indic <- (.tmp$time<=pro.time)

   return(.tmp$surv[.tmp$indic][sum(.tmp$indic)]) } }

.survie.temp<-sapply(1:.n, FUN = ".survie.x")

.survie.lambda<-function(x) {
 mean(.survie.temp * (.temp.data$variable>x), na.rm=TRUE) }

.survie.marginale<-mean(.survie.temp, na.rm=TRUE)

.temp.se<-function(x) {
 (1-sum(.temp.data$variable<=x)/.n-.survie.lambda(x))/(1-.survie.marginale) }
 
.temp.sp<-function(x)
{ 1-(.survie.lambda(x)/.survie.marginale) }

.tab.res<-data.frame(
   cut.off=cut.off,
   se=suppressWarnings(sapply(cut.off, FUN=".temp.se")),
   sp1=1-suppressWarnings(sapply(cut.off, FUN=".temp.sp")))

.tab.res$se[.tab.res$se>1]<-NA
.tab.res$sp1[.tab.res$sp1>1]<-NA

.tab.res<-.tab.res[!is.na(.tab.res$sp1 + .tab.res$se),]

.tab.res.temp<-.tab.res[!is.na(.tab.res$sp1 + .tab.res$se),]

.tab.res.temp<-.tab.res.temp[order(.tab.res.temp$sp1, .tab.res.temp$se),]

.tab.res.temp<-rbind(c(NA, 0, 0), .tab.res.temp, c(NA, 1, 1))

.tab.res$sp<-1-.tab.res$sp1



return(list(
table = .tab.res[,c("cut.off", "se", "sp")],
auc = sum((.tab.res.temp$sp1[2:length(.tab.res.temp$sp1)] - .tab.res.temp$sp1[1:(length(.tab.res.temp$sp1)-1)]) * 0.5 * (.tab.res.temp$se[2:length(.tab.res.temp$sp1)] + .tab.res.temp$se[1:(length(.tab.res.temp$sp1)-1)])),
missing = .n.na,
warning = .warnings ) )

}
}
}


EUt<-function (times, failures, variable, pro.time, cut.off, estimator = "naive", 
    lambda = NULL, u.tn, u.fp, u.tp0, u.tp1, u.fn0, u.fn1, full.efficacy = FALSE, 
    gain = 0) {

if (sum(estimator != c("kaplan-meier", "akritas", "naive"))==3) {
  stop("The estimator have to be selected among the three following possibilities: 'kaplan-meier', 'akritas', 'naive'.")  }

if (!is.null(lambda) & estimator != "akritas") {
  stop("The parameter lambda should be only defined for the Akritas estimator.") }

if (is.null(lambda) & estimator == "akritas") {
  stop("The parameter lambda have to be defined.") }

.temp.data <- data.frame(times = times, failures = failures, 
        variable = variable)

.temp.data$failures[.temp.data$times > pro.time] <- 0
.temp.data$times[.temp.data$times > pro.time] <- pro.time + 0.01

.n.na <- sum(is.na(.temp.data$times + .temp.data$failures + 
        .temp.data$variable))

.temp.data <- .temp.data[!is.na(.temp.data$times + .temp.data$failures + 
        .temp.data$variable), ]

.n <- dim(.temp.data)[1]

mean.LR <- function(x) {

	.data<-.temp.data[.temp.data$variable<=x, ]

	if(dim(.data)[1]==0) {return (NA)} else{

	if( (max(.data$times)<pro.time) & (.data$failure[.data$times==max(.data$times)]==0) ) {
		return (NA)	} else {
        .km <- summary(survfit(Surv(times, failures) ~ 1, data = .data))
    	.t <- c(0, .km$time[.km$time <= pro.time], min(pro.time, max(.data$times)))
        .s <- c(1, .km$surv[.km$time <= pro.time], .km$surv[length(.km$surv)])
       
        return( sum((.t[2:length(.t)] - .t[1:(length(.t) - 1)]) * 
            .s[1:(length(.s) - 1)])	)

	}	}   }

mean.HR <- function(x) {

	.data<-.temp.data[.temp.data$variable>x, ]

	if(dim(.data)[1]==0) {return (NA)} else{

	if( (max(.data$times)<pro.time) & (.data$failure[.data$times==max(.data$times)]==0) ) {
       	return (NA)	} else {
         .km <- summary(survfit(Surv(times, failures) ~ 1, data = .data))
         .t <- c(0, .km$time[.km$time <= pro.time], min(pro.time, max(.data$times)))
         .s <- c(1, .km$surv[.km$time <= pro.time], .km$surv[length(.km$surv)])
		 
         return( sum((.t[2:length(.t)] - .t[1:(length(.t) - 1)]) * 
            .s[1:(length(.s) - 1)] ) )
	
	} 	}    }

.eLR <- sapply(cut.off, FUN = "mean.LR")
.eLR[cut.off<=max(cut.off[is.na(.eLR)])]<-NA
	
.eHR <- pmin((1 + gain) * sapply(cut.off, FUN = "mean.HR"), pro.time)
.eHR[cut.off>=min(cut.off[is.na(.eHR)])]<-NA

if (full.efficacy == TRUE) { .eHR <- rep(pro.time, length(cut.off)) }

.q.tn <- rep(u.tn * pro.time, length(cut.off))
.q.fp <- rep(u.fp * pro.time, length(cut.off))
.q.tp <- u.tp0 * .eHR + u.tp1 * (pro.time - .eHR)
.q.fn <- u.fn0 * .eLR + u.fn1 * (pro.time - .eLR)

mean.t <- function(times, failures, pro.time) {
	.km <- summary(survfit(Surv(times, failures) ~ 1))
    .t <- c(0, .km$time[.km$time <= pro.time], min(pro.time, max(times)))
    .s <- c(1, .km$surv[.km$time <= pro.time], .km$surv[length(.km$surv)])
       
    return(	sum((.t[2:length(.t)] - .t[1:(length(.t) - 1)])
    	* .s[1:(length(.s) - 1)]) )  }
	
.e.nob.ttt<-mean.t(.temp.data$times, .temp.data$failures, pro.time)
	
.e.all.ttt<-min(pmin((1 + gain) * .e.nob.ttt), pro.time)

if (full.efficacy == TRUE) { .e.all.ttt <- pro.time }

.surv <- summary(survfit(Surv(times, failures) ~ 1, data = .temp.data))
.surv <- .surv$surv[length(.surv$surv)]

.c.nob.ttt<-max(variable)
.q.nob.p <- u.fn0 * .e.nob.ttt + u.fn1 * (pro.time - .e.nob.ttt) # nobody treated
eu.nob.ttt<- .q.tn[1] * .surv + .q.nob.p * (1-.surv)

.c.all.ttt<-min(variable)
.q.all.p <- u.tp0 * .e.all.ttt + u.tp1 * (pro.time - .e.all.ttt) # all patients treated
eu.all.ttt <- .q.fp[1] * .surv + .q.all.p * (1-.surv)

if (estimator == "akritas")
{
   .temp.data <- .temp.data[order(.temp.data$variable),]
   .pas <- round(lambda * length(.temp.data$times))
   
   .survie.x <- function(x) {
        .tmp <- summary(survfit(Surv(times, failures) ~ 1, 
          data = .temp.data[max(1, (x - .pas)):min((x+.pas), .n), ]))
        return(min(0.99999, .tmp$surv[.tmp$time <= pro.time][sum(.tmp$time <= pro.time)])) }
		
   .survie.temp <- sapply(1:.n, FUN = ".survie.x")
   
   .survie.lambda <- function(x) {
            sum(.survie.temp * (.temp.data$variable > x))/.n }

   .survie.marginale <- sum(.survie.temp)/.n
   
   .emp.disti <- function(x) { sum(.temp.data$variable > x)/.n }
   
   .p.fp <- sapply(cut.off, FUN = ".survie.lambda")
   .p.tp <- sapply(cut.off, FUN = ".emp.disti") - .p.fp
   .p.tn <- .survie.marginale - .p.fp
   .p.fn <- 1 - .p.tp - .p.fp - .p.tn
        
   .exp.utility <- .p.tp*.q.tp + .p.fp*.q.fp + .p.fn*.q.fn + .p.tn*.q.tn

   .tab.res <- data.frame(cut.off = cut.off, utility = .exp.utility)
   .prob.res<-data.frame(cut.off = cut.off, tp = .p.tp, fp = .p.fp, tn = .p.tn, fn = .p.fn)
   .qaly.res<-data.frame(cut.off = cut.off, tp = .q.tp, fp = .q.fp, tn = .q.tn, fn = .q.fn)

   .prob.res[is.na(.tab.res$utility),]<-NA	
   .qaly.res[is.na(.tab.res$utility),]<-NA	

	.est<- .tab.res$cut.off[max(.tab.res$utility, na.rm=TRUE) == .tab.res$utility & !is.na(.tab.res$utility)]
	.m<-round(max(.tab.res$utility,na.rm=T),10)
	
	
    if(.m<=max(c(eu.nob.ttt,eu.all.ttt))) {
	  .m <-max(c(eu.nob.ttt,eu.all.ttt))
	  if(eu.nob.ttt<eu.all.ttt) {.est<-.c.all.ttt}
    	  else{.est<-.c.nob.ttt} }

    return(list(
	missing = .n.na,
	proba.table = .prob.res,
	qaly.table = .qaly.res,
	eu.table = .tab.res,
	eu.all =  eu.all.ttt,
	eu.no =  eu.nob.ttt,
	eu.maximum = .m,
	estimation = .est))
}

if (estimator == "naive")
{
    km.c <- summary(survfit(Surv(.temp.data$times, 1*(.temp.data$failures==0)) ~ 1))
    km.c <- data.frame(time = c(0, km.c$time), surv = c(1, km.c$surv))
    km.c$surv[km.c$surv == 0] <- 0.0001

    s.c <- sapply(.temp.data$times, FUN = function(x) {
      km.c$surv[km.c$time <= x][sum(km.c$time <= x)] } )

    p1.sup <- function(x) {
      sum(1 * ((.temp.data$variable > x) & (.temp.data$times <= pro.time)) *
	  .temp.data$failures/s.c)/.n }
			
    p0.sup <- function(x) {
      sum(1 * (.temp.data$variable > x) - 1 * ((.temp.data$variable > x) &
	  (.temp.data$times <= pro.time)) * .temp.data$failures/s.c)/.n }
			
    p1.inf <- function(x) {
      sum(1 * ((.temp.data$variable <= x) & (.temp.data$times <= pro.time)) *
	  .temp.data$failures/s.c)/.n }
	  
    .p.tp <- sapply(cut.off, FUN = "p1.sup")
    .p.fp <- sapply(cut.off, FUN = "p0.sup")
    .p.fn <- sapply(cut.off, FUN = "p1.inf")
    .p.tn <- 1 - .p.tp - .p.fp - .p.fn

        
   .exp.utility <- .p.tp*.q.tp + .p.fp*.q.fp + .p.fn*.q.fn + .p.tn*.q.tn

   .tab.res <- data.frame(cut.off = cut.off, utility = .exp.utility)
   .prob.res<-data.frame(cut.off = cut.off, tp = .p.tp, fp = .p.fp, tn = .p.tn, fn = .p.fn)
   .qaly.res<-data.frame(cut.off = cut.off, tp = .q.tp, fp = .q.fp, tn = .q.tn, fn = .q.fn)

   .prob.res[is.na(.tab.res$utility),]<-NA	
   .qaly.res[is.na(.tab.res$utility),]<-NA	

	.est<- .tab.res$cut.off[max(.tab.res$utility, na.rm=TRUE) == .tab.res$utility & !is.na(.tab.res$utility)]
	.m<-round(max(.tab.res$utility,na.rm=T),10)
	
	
    if(.m<=max(c(eu.nob.ttt,eu.all.ttt))) {
	  .m <-max(c(eu.nob.ttt,eu.all.ttt))
	  if(eu.nob.ttt<eu.all.ttt) {.est<-.c.all.ttt}
    	  else{.est<-.c.nob.ttt} }

    return(list(
	missing = .n.na,
	proba.table = .prob.res,
	qaly.table = .qaly.res,
	eu.table = .tab.res,
	eu.all =  eu.all.ttt,
	eu.no =  eu.nob.ttt,
	eu.maximum = .m,
	estimation = .est))
}

if (estimator == "kaplan-meier") {
    .surv.total <- summary(survfit(Surv(times, failures) ~ 1, data = .temp.data))
    .surv.total$indic <- (.surv.total$time <= pro.time)
	
    p1.sup <- function(x) {
      .surv.cond <- try(summary(survfit(Surv(times, failures) ~ 1,
	    data = .temp.data[.temp.data$variable > x,])), silent = TRUE)
      if (inherits(.surv.cond, "try-error")) {return(NA)} else {
        .surv.cond <- data.frame(time = c(1 - 0.99999999999, .surv.cond$time),
		  surv = c(0.99999999999, .surv.cond$surv))
        .surv.cond$surv[.surv.cond$surv == 0] <- 1 - 0.99999999999
        .surv.cond$indic <- (.surv.cond$time <= pro.time)
        return((1 - .surv.cond$surv[.surv.cond$indic][sum(.surv.cond$indic)]) * 
          (1 - (sum(.temp.data$variable <= x)/length(times)))) }  }

    p0.sup <- function(x) {
      .surv.cond <- try(summary(survfit(Surv(times, failures) ~ 1,
	    data = .temp.data[.temp.data$variable > x,])), silent = TRUE)
      if (inherits(.surv.cond, "try-error")) {return(NA)} else {
        .surv.cond <- data.frame(time = c(1 - 0.99999999999, .surv.cond$time),
		  surv = c(0.99999999999, .surv.cond$surv))
        .surv.cond$surv[.surv.cond$surv == 0] <- 1 - 0.99999999999
        .surv.cond$indic <- (.surv.cond$time <= pro.time)
        return(.surv.cond$surv[.surv.cond$indic][sum(.surv.cond$indic)] * 
          (1 - (sum(.temp.data$variable <= x)/length(times))) ) } }

    p1.inf <- function(x) {
      .surv.cond <- try(summary(survfit(Surv(times, failures) ~ 1,
		data = .temp.data[.temp.data$variable <= x,])), silent = TRUE)
      if (inherits(.surv.cond, "try-error")) {return(NA)} else {
        .surv.cond <- data.frame(time = c(1 - 0.99999999999, .surv.cond$time),
   		  surv = c(0.99999999999, .surv.cond$surv))
        .surv.cond$surv[.surv.cond$surv == 0] <- 1 - 0.99999999999
        .surv.cond$indic <- (.surv.cond$time <= pro.time)
        return((1 - .surv.cond$surv[.surv.cond$indic][sum(.surv.cond$indic)]) * 
                  (sum(.temp.data$variable <= x)/length(times)) ) } }

    .p.tp <- sapply(cut.off, FUN = "p1.sup")
    .p.fp <- sapply(cut.off, FUN = "p0.sup")
    .p.fn <- sapply(cut.off, FUN = "p1.inf")
    .p.tn <- 1 - .p.tp - .p.fp - .p.fn
       
    .exp.utility <- .p.tp*.q.tp + .p.fp*.q.fp + .p.fn*.q.fn + .p.tn*.q.tn

   .tab.res <- data.frame(cut.off = cut.off, utility = .exp.utility)
   .prob.res<-data.frame(cut.off = cut.off, tp = .p.tp, fp = .p.fp, tn = .p.tn, fn = .p.fn)
   .qaly.res<-data.frame(cut.off = cut.off, tp = .q.tp, fp = .q.fp, tn = .q.tn, fn = .q.fn)

   .prob.res[is.na(.tab.res$utility),]<-NA	
   .qaly.res[is.na(.tab.res$utility),]<-NA	

	.est<- .tab.res$cut.off[max(.tab.res$utility, na.rm=TRUE) == .tab.res$utility & !is.na(.tab.res$utility)]
	.m<-round(max(.tab.res$utility,na.rm=T),10)
	
	
    if(.m<=max(c(eu.nob.ttt,eu.all.ttt))) {
	  .m <-max(c(eu.nob.ttt,eu.all.ttt))
	  if(eu.nob.ttt<eu.all.ttt) {.est<-.c.all.ttt}
    	  else{.est<-.c.nob.ttt} }

    return(list(
	missing = .n.na,
	proba.table = .prob.res,
	qaly.table = .qaly.res,
	eu.table = .tab.res,
	eu.all =  eu.all.ttt,
	eu.no =  eu.nob.ttt,
	eu.maximum = .m,
	estimation = .est))

}
}
